#!/usr/bin/env perl

$num_args = $#ARGV;
if ($num_args != 4) {
  print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n");
  exit;
}
if($ARGV[0] !~ m/^(\d\d?\d?).(\d\d?\d?).(\d\d?\d?).(\d\d?\d?)$/ ) {
	print("Your IP is strange...\n");
	print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n");
	exit;
}
if($ARGV[1] =~ m/[^a-zA-Z0-9]/) {
	print("It's not a good idea to use special character in your community name, but whatever...\n");
	print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n\n");
}
if($ARGV[2] ne 'in' && $ARGV[2] ne 'out' && $ARGV[2] ne 'both' ) {
	print("in/out/both seems to be wrong...\n");
	print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n");
	exit;
}
if($ARGV[3] !~ /^(\d+\.?\d*|\.\d+)$/ ) {
	print("[Time] seems to be wrong...\n");
	print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n");
	exit;
}
if($ARGV[4] !~ /^(\d+\.?\d*|\.\d+)$/ ) {
	print("[Interface ID] seems to be wrong...\n");
	print("Usage: ifInfo.pl [\@IP] [community] in/out/both [Time] [Interface ID]\n");
	exit;
}
my $ipaddr=$ARGV[0];
my $community=$ARGV[1];
my $direction=$ARGV[2];
my $howlong=$ARGV[3];
my $id=$ARGV[4];


print "==========================================\n";
print "Interface $ARGV[0] (0 sec)\n";
print "==========================================\n";
if( $direction eq 'in' || $direction eq 'both' ) {
	my $ifHCInUcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.7.$id`;
	@inUnicastStart = split(' ', $ifHCInUcastPkts);
	my $ifHCInMulticastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.8.$id`;
	@inMulticastStart = split(' ', $ifHCInMulticastPkts);
	my $ifHCInBroadcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.9.$id`;
	@inBroadcastStart = split(' ', $ifHCInBroadcastPkts);
	print "ifHCInUcastPkts : ".@inUnicastStart[1]."\n";
	print "ifHCInMulticastPkts : ".@inMulticastStart[1]."\n";
	print "ifHCInBroadcastPkts : ".@inBroadcastStart[1]."\n";
	print "Broadcast rate (in) : ";
	if(@inUnicastStart[1]+@inMulticastStart[1]+@inBroadcastStart[1]!=0)
	{	
		print @inBroadcastStart[1]/(@inUnicastStart[1]+@inMulticastStart[1]+@inBroadcastStart[1])*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
}
if( $direction eq 'out' || $direction eq 'both' ) {
	my $ifHCOutUcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.11.$id`;
	@outUnicastStart = split(' ', $ifHCOutUcastPkts);
	my $ifHCOutMulticastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.12.$id`;
	@outMulticastStart = split(' ', $ifHCOutMulticastPkts);
	my $ifHCOutBroadcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.13.$id`;
	@outBroadcastStart = split(' ', $ifHCOutBroadcastPkts);
	print "ifHCOutUcastPkts : ".@outUnicastStart[1]."\n";
	print "ifHCOutMulticastPkts : ".@outMulticastStart[1]."\n";
	print "ifHCOutBroadcastPkts : ".@outBroadcastStart[1]."\n";
	print "Broadcast rate (out) : ";
	if(@outUnicastStart[1]+@outMulticastStart[1]+@outBroadcastStart[1]!=0)
	{
		print @outBroadcastStart[1]/(@outUnicastStart[1]+@outMulticastStart[1]+@outBroadcastStart[1])*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
}
if ($direction eq 'both')
{
	$bothUnicastStart=@inUnicastStart[1]+@outUnicastStart[1];
	$bothMulticastStart=@inMulticastStart[1]+@outMulticastStart[1];
	$bothBroadcastStart=@inBroadcastStart[1]+@outBroadcastStart[1];
	print "Unicast (in+out) : ".$bothUnicastStart."\n";
	print "Multicast (in+out) : ".$bothMulticastStart."\n";
	print "Broadcast (in+out) : ".$bothBroadcastStart."\n";
	print "Broadcast rate (in+out) : ";
	if($bothUnicastStart+$bothMulticastStart+$bothBroadcastStart!=0)
	{
		print $bothBroadcastStart/($bothUnicastStart+$bothMulticastStart+$bothBroadcastStart)*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
}

sleep($howlong);

print "==========================================\n";
print "Interface $ARGV[0] (".$howlong."sec)\n";
print "==========================================\n";
if( $direction eq 'in' || $direction eq 'both' ) {
	my $ifHCInUcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.7.$id`;
	@inUnicastEnd = split(' ', $ifHCInUcastPkts);
	my $ifHCInMulticastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.8.$id`;
	@inMulticastEnd = split(' ', $ifHCInMulticastPkts);
	my $ifHCInBroadcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.9.$id`;
	@inBroadcastEnd = split(' ', $ifHCInBroadcastPkts);
	print "ifHCInUcastPkts : ".@inUnicastEnd[1]."\n";
	print "ifHCInMulticastPkts : ".@inMulticastEnd[1]."\n";
	print "ifHCInBroadcastPkts : ".@inBroadcastEnd[1]."\n";
	print "Broadcast rate : ";
	if(@inUnicastEnd[1]+@inMulticastEnd[1]+@inBroadcastEnd[1]!=0) {
		print @inBroadcastEnd[1]/(@inUnicastEnd[1]+@inMulticastEnd[1]+@inBroadcastEnd[1])*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
}
if( $direction eq 'out' || $direction eq 'both' ) {
	my $ifHCOutUcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.11.$id`;
	@outUnicastEnd = split(' ', $ifHCOutUcastPkts);
	my $ifHCOutMulticastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.12.$id`;
	@outMulticastEnd = split(' ', $ifHCOutMulticastPkts);
	my $ifHCOutBroadcastPkts = `snmpwalk -v2c -c $community -O v $ipaddr  .1.3.6.1.2.1.31.1.1.1.13.$id`;
	@outBroadcastEnd = split(' ', $ifHCOutBroadcastPkts);
	print "ifHCOutUcastPkts : ".@outUnicastEnd[1]."\n";
	print "ifHCOutMulticastPkts : ".@outMulticastEnd[1]."\n";
	print "ifHCOutBroadcastPkts : ".@outBroadcastEnd[1]."\n";
	print "Broadcast rate : ";
	if(@outUnicastEnd[1]+@outMulticastEnd[1]+@outBroadcastEnd[1]!=0) {
		print @outBroadcastEnd[1]/(@outUnicastEnd[1]+@outMulticastEnd[1]+@outBroadcastEnd[1])*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
}
if ($direction eq 'both')
{
	$bothUnicastEnd=@inUnicastEnd[1]+@outUnicastEnd[1];
	$bothMulticastEnd=@inMulticastEnd[1]+@outMulticastEnd[1];
	$bothBroadcastEnd=@inBroadcastEnd[1]+@outBroadcastEnd[1];
	print "ifHCInUcastPkts : ".$bothUnicastEnd."\n";
	print "ifHCInMulticastPkts : ".$bothMulticastEnd."\n";
	print "ifHCInBroadcastPkts : ".$bothBroadcastEnd."\n";
	print "Broadcast rate : ";
	if($bothUnicastEnd+$bothMulticastEnd+$bothBroadcastEnd!=0) {
		print $bothBroadcastEnd/($bothUnicastEnd+$bothMulticastEnd+$bothBroadcastEnd)*100;
		print "\n\n";
	}
	else {
		print 0;
		print "\n\n";
	}
	
}


if($direction eq 'in') {
	$unicast=@inUnicastEnd[1]-@inUnicastStart[1];
	$multicast=@inMulticastEnd[1]-@inMulticastStart[1];
	$broadcast=@inBroadcastEnd[1]-@inBroadcastStart[1];
}
elsif($direction eq 'out') {
	$unicast=@outUnicastEnd[1]-@outUnicastStart[1];
	$multicast=@outMulticastEnd[1]-@outMulticastStart[1];
	$broadcast=@outBroadcastEnd[1]-@outBroadcastStart[1];
}
elsif($direction eq 'both') {
	$unicast=$bothUnicastEnd-$bothUnicastStart;
	$multicast=$bothMulticastEnd-$bothMulticastStart;
	$broadcast=$bothBroadcastEnd-$bothBroadcastStart;
}
print "==========================================\n";
print "=== Interface $ARGV[0] (".$howlong." sec ratio)===\n";
print "==========================================\n";
print "ifHCInUcastPkts : ".$unicast."\n";
print "ifHCInMulticastPkts : ".$multicast."\n";
print "ifHCInBroadcastPkts : ".$broadcast."\n";
print "Broadcast rate : ";
if($unicast+$multicast+$broadcast!=0) {
	print $broadcast/($unicast+$multicast+$broadcast)*100;
}
else {
	print '0';
}
print "\n";
